***		AEJMacro2013-0231_Feenstra_AggregateEstimates								***
***																					***
***	This program:																	***
***		Estimates an aggregate substitution elasticity using Feenstra (1994) on		***
***		aggregate data		 														***
***		Compute the aggregate trade elasticity										***
***	Input: data.temp_agbil.sas7bdat (aggregate-level estimation database)			***
***	       data.weight_IO_MO_wkva.sas7bdat (weight matrix)		                	***
***	Output: res.sol_a_LaspeyresW.sas7bdat Aggregate trade elasticity)				***	;


goptions devmap=winansi keymap=winansi;
options pagesize=100 linesize=100;
OPTIONS OBS=10000 NOREPLACE; *SAS won't process any data, not replace any existing data;
OPTIONS OBS=MAX REPLACE; *SAS will proceed;


libname data "W:\Elast_JI\Output\AEJ\AdditionalMaterial\Feenstra";* where the raw data are saved;
libname res "W:\Elast_JI\Output\AEJ\AdditionalMaterial\Feenstra\Output";*where the output is saved;


%let prog= W:\Elast_JI\Output\AEJ\AdditionalMaterial\Feenstra;* where the programs are saved;
%let sect= isic3;*sectoral level ISIC rev3; 

/**********************************************************************************************/

* import data;
data temp_agbil;
	set data.temp_agbil;
data weight;
	set data.weight_IO_MO_wkva;
run;

* first-stage estimation;
proc glm noprint data=temp_agbil;
	class i;
	model dls2 dlps =i /noint p  nouni;
	output out=base_ag2 p=s2hat pshat;
run;quit;

* second-stage estimation without heteroscedasticity correction + residual computation;
proc glm data=base_ag2;
	model dlp2=s2hat pshat  / noint solution  ; 
	ods output ParameterEstimates=work.sol_ag ;
run;quit;

data s2hat;
	set sol_ag;
	where Parameter="s2hat";
	coef_s2hat=estimate;
	merge=1;
data pshat;
	set sol_ag;
	where Parameter="pshat";
	coef_pshat=estimate;
	merge=1;
data base_ag2;
	set base_ag2;
	merge=1;
proc sort data=base_ag2;
	by merge;
proc sort data=s2hat;
	by merge;
proc sort data=pshat;
	by merge;
data base_ag2;
	merge base_ag2 s2hat(keep=coef_s2hat merge) pshat(keep=coef_pshat merge) ;
	by merge;* add estimated coef in base;
	res=dlp2-coef_s2hat*dls2-coef_pshat*dlps;
	res2=res*res;
run;

* instrument residuals using country-specific dummies (no sectorial dimension);
proc glm noprint data=base_ag2;
	class i;
	model res2=i/p nouni;
	output out=base_ag2 p=r2hat;
run;quit;

* Correct for heteroscedasticity;
data base_ag2;
	set base_ag2;
	rhat=sqrt(r2hat);
	Y=dlp2/rhat;
	X1=s2hat/rhat;
	X2=pshat/rhat;
run;

* estimation with heteroscedasticity correction ;
ods output parameterEstimates=sol_ag invXPX=var_ag FitStatistics=rmse_ag;
proc glm data=base_ag2;
	model Y= X1 X2 /noint solution inverse;
run;quit;
proc sql noprint;
	select RootMSE**2 INTO : s2
	from rmse_ag;
quit;
data var_ag;
	set var_ag;
	array valeurs _NUMERIC_;
	do over valeurs;
	valeurs =valeurs *&s2;
	end;
run;

* Compute rho, omega and sigma from theta1 and theta2;
data X1;
	set sol_ag;
	where parameter="X1";
	rename estimate=X1;
	VARX1=StdErr*StdErr;
	merge=1;
data X2;
	set sol_ag;
	where parameter="X2";
	rename estimate=X2;
	VARX2=StdErr*StdErr;
	merge=1;
data COVX1X2;
	set var_ag;
	where parameter='X1';
	COVX1X2=X2;
	merge=1;
data COVX1X2;
	set COVX1X2;
	keep COVX1X2 merge;
proc sort data=COVX1X2;
	by merge;
proc sort data=X1;
	by merge;
proc sort data=X2;
	by merge;
data sigma_ag;
	merge X1(keep=merge X1 VARX1) X2(keep=merge X2 VARX2) COVX1X2 ;
 	by merge;
data sigma_ag;
	set sigma_ag;
	tX1=abs(X1/(VARX1**0.5));
	tX2=abs(X2/(VARX2**0.5));
	if tX1<1.67 then X1=0;
	if tX2<1.67 then X2=0;
	rac=sqrt(X2*X2+4*X1);
	if X1>0 then epsilon=(X2+rac)/(-2*X1);
	omega=-epsilon*X1;
	domdep=-X1;
	domdX1=-epsilon;
	depdX1=(X2*X2+2*X1+X2*rac)/(2*X1*X1*rac);
	depdX2=(rac+X2)/(-2*X1*rac);
	COVepsilonX1=depdX1*VARX1+depdX2*COVX1X2;
	VARepsilon=(depdX1*depdX1)*VARX1+2*depdX1*depdX2*COVX1X2+(depdX2*depdX2)*VARX2;
	VARomega=(domdep*domdep)*VARepsilon+2*domdep*domdX1*COVepsilonX1+(domdX1*domdX1)*VARX1;
run;

* Compute Aggregate trade elasticity;
data weight;
	set weight;
	wkmwk=wk*wKM;
	drop nk;
proc univariate noprint data=weight;
	var wkmwk ;
	output out=sumwkmwk sum=sumwkmwk ;
run;
data sumwkmwk;
	set sumwkmwk;
	weight=1;
proc sort data=sumwkmwk;
	by weight;
data weight;
	set weight;
	weight=1;
proc sort data=weight;
	by weight;
data weight;
	merge weight sumwkmwk;
	by weight;
	nk=wk*wkm/sumwkmwk;
proc univariate noprint data=weight;
	by weight;
	var nk ;
	output out=sumnk sum=sumnk ;
proc sort data=sumnk;
	by weight;
proc sort data=weight;
	by weight;
data weight;
	merge weight sumnk;
	by weight;
data weight;
	set weight;
	nk=nk/sumnk;
	nkwkm=nk*wKM;
proc univariate noprint data=weight;
	where nk^=.;
	var nkwkm  ;
	output out=sigma_c sum= nkwkm ;
run;
data sol_a;
	merge sigma_c sigma_ag(keep=epsilon VARepsilon X1 X2 varX1 varX2);
	sigma=1-epsilon;
	se_epsilon=sqrt( VARepsilon);
	elimpt_tot=1-sigma+(sigma-1)*nkwkm;
	var_elimpt_tot=(-1+nkwkm)**2*VARepsilon;
	se_elimpt_tot=sqrt(var_elimpt_tot);
run;
proc printto ;run;
data res.sol_a_LaspeyresW;
	set sol_a;
	keep epsilon se_epsilon elimpt_tot se_elimpt_tot  ;
run;
