***			AEJMacro2013-0231_Feenstra_PooledMicroEstimates_sectorIV						***
***																							***
***	This program:																			***
***		Estimates a constrained substitution elasticity using Feenstra (1994) and the 		***
***		constraint that coefficients are homogeneous across sectors		 					***
***		Compute the aggregate trade elasticity												***
***	Input: data.temp_bil.sas7bdat (product-level estimation database)						***
***	       data.weight_IO_MO_wkva.sas7bdat (weight matrix)		                			***
***	       prog.panel_hs6_c_nohetero.sas (estimation without the heteroscedasticity 		***
***		correction)																			***
***	       prog.panel_hs6_c_hetero.sas (estimation with the	heteroscedasticity correction)	***
***	Output: res.sol_c_sectorIV.sas7bdat (Constrained trade elasticity)								***	;

goptions devmap=winansi keymap=winansi;
options pagesize=100 linesize=100;
OPTIONS OBS=10000 NOREPLACE; *SAS won't process any data, not replace any existing data;
OPTIONS OBS=MAX REPLACE; *SAS will proceed;


libname data "W:\Elast_JI\Output\AEJ\AdditionalMaterial\Feenstra";* where the raw data are saved;
libname res "W:\Elast_JI\Output\AEJ\AdditionalMaterial\Feenstra\Output";*where the output is saved;


%let prog= W:\Elast_JI\Output\AEJ\AdditionalMaterial\Feenstra;* where the programs are saved;
%let sect= isic3;*sectoral level ISIC rev3; 

/**********************************************************************************************/

* import data;
data temp_bil;
	set data.temp_bil;
data weight;
	set data.weight_IO_MO_wkva;
run;

* first-stage estimation;
proc delete data=base_c;
run;
proc glm noprint data=temp_bil;
	class i &sect;
	model dls2 dlps =i*&sect /noint p  nouni;
	output out=base_c p=s2hat pshat;
run;quit;

* estimation without heteroscedasticity correction + residual computation;
%let base=base_c;
%let sol=sol_c_nohetero;
proc printto log="nul:";run;
proc printto file="nul:";run;
%include "&prog\panel_hs6_c_nohetero.sas";/* panel with hs6 fixed effects*/
proc printto;run;

* instrument residuals using country-specific dummies (no sectorial dimension);
proc glm noprint data=base_c;
	class i &sect;
	model res2=i*&sect/p nouni;
	output out=base_c p=r2hat;
run;quit;

* Correct for heteroscedasticity;
data base_c;
	set base_c;
	rhat=sqrt(r2hat);
	Y=dlp2/rhat;
	X0=1/rhat;
	X1=s2hat/rhat;
	X2=pshat/rhat;
run;

* estimation with heteroscedasticity correction;
%let basesol=work.sol_c;
%let basevar=work.var_cov_c;
%let basermse=work.rmse_c;
proc printto log="nul:";run;
proc printto file="nul:";run;
 %include "&prog\panel_hs6_c_hetero.sas";/*panel with hs6 constantes*/
proc printto;run;

* Compute sigma and elasticity;
data varX1;
	set var_cov_c;
	where Parameter="X1";
	varX1=X1;
data varX1;
	set varX1;
	keep varX1;
data varX2;
	set var_cov_c;
	where Parameter="X2";
	varX2=X2;
data varX2;
	set varX2;
	keep varX2;
data covX1X2;
	set var_cov_c;
	where Parameter="X1";
	covX1X2=X2;
data covX1X2;
	set covX1X2;
	keep covX1X2;
data X1;
	set sol_c;
	where Parameter="X1";
	X1=Estimate;
data X1;
	set X1;
	keep X1;
data X2;
	set sol_c;
	where Parameter="X2";
	X2=Estimate;
data X2;
	set X2;
	keep X2;
data sol_con;
	merge X1 X2 varX1 varX2 covX1X2;
	tX1=abs(X1/(VARX1**0.5));
	tX2=abs(X2/(VARX2**0.5));
	if tX1<1.67 then X1=0;
	if tX2<1.67 then X2=0;
	rac=sqrt(X2*X2+4*X1);
	if X1>0 then epsilon=(X2+rac)/(-2*X1);
	omega=-epsilon*X1;
	domdep=-X1;
	domdX1=-epsilon;
	depdX1=(X2*X2+2*X1+X2*rac)/(2*X1*X1*rac);
	depdX2=(rac+X2)/(-2*X1*rac);
	COVepsilonX1=depdX1*VARX1+depdX2*COVX1X2;
	VARepsilon=(depdX1*depdX1)*VARX1+2*depdX1*depdX2*COVX1X2+(depdX2*depdX2)*VARX2;
	VARomega=(domdep*domdep)*VARepsilon+2*domdep*domdX1*COVepsilonX1+(domdX1*domdX1)*VARX1;
run;

* Compute Aggregate trade elasticity;
data weight;
	set weight;
	wkmwk=wk*wKM;
	drop nk;
proc univariate noprint data=weight;
	var wkmwk ;
	output out=sumwkmwk sum=sumwkmwk ;
run;
data sumwkmwk;
	set sumwkmwk;
	weight=1;
proc sort data=sumwkmwk;
	by weight;
data weight;
	set weight;
	weight=1;
proc sort data=weight;
	by weight;
data weight;
	merge weight sumwkmwk;
	by weight;
	nk=wk*wkm/sumwkmwk;
proc univariate noprint data=weight;
	by weight;
	var nk ;
	output out=sumnk sum=sumnk ;
proc sort data=sumnk;
	by weight;
proc sort data=weight;
	by weight;
data weight;
	merge weight sumnk;
	by weight;
data weight;
	set weight;
	nk=nk/sumnk;
	nkwkm=nk*wKM;
proc univariate noprint data=weight;
	where nk^=.;
	var nkwkm  ;
	output out=sigma_c sum= nkwkm ;
run;
data sol_c;
	merge sigma_c sol_con(keep=epsilon VARepsilon X1 X2 varX1 varX2);
	sigma=1-epsilon;
	se_epsilon=sqrt(VARepsilon);
	elimpt_tot=1-sigma+(sigma-1)*nkwkm;
	var_elimpt_tot=(-1+nkwkm)**2*VARepsilon;
	se_elimpt_tot=sqrt(var_elimpt_tot);
run;
proc printto ;run;
data res.sol_c_sectorIV;
	set sol_c;
	keep epsilon se_epsilon elimpt_tot se_elimpt_tot;
run;
run;

